//  Copyright (C) 2018 Zilliqa
//
//  This file is part of zilliqa-js
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <https://www.gnu.org/licenses/>.
import { __assign, __awaiter, __generator, __read, __spreadArray, __values } from "tslib";
import { RPCMethod } from "@zilliqa-js/core";
import { bytes, validation, Long } from "@zilliqa-js/util";
import * as proto from "@zilliqa-js/proto";
var ZilliqaMessage = proto.ZilliqaMessage;
import { Buffer } from "buffer"; /* tslint:disable:no-unused-variable */
export var encodeTransactionProto = function (tx) {
    var msg = {
        version: tx.version,
        nonce: new Long(tx.nonce || 0, 0),
        // core protocol Schnorr expects lowercase, non-prefixed address.
        toaddr: bytes.hexToByteArray(tx.toAddr.replace("0x", "").toLowerCase()),
        senderpubkey: ZilliqaMessage.ByteArray.create({
            data: bytes.hexToByteArray(tx.pubKey || "00"),
        }),
        amount: ZilliqaMessage.ByteArray.create({
            data: Uint8Array.from(tx.amount.toArrayLike(Buffer, undefined, 16)),
        }),
        gasprice: ZilliqaMessage.ByteArray.create({
            data: Uint8Array.from(tx.gasPrice.toArrayLike(Buffer, undefined, 16)),
        }),
        gaslimit: tx.gasLimit,
        code: tx.code && tx.code.length
            ? Uint8Array.from(__spreadArray([], __read(tx.code), false).map(function (c) { return c.charCodeAt(0); }))
            : null,
        data: tx.data && tx.data.length
            ? Uint8Array.from(__spreadArray([], __read(tx.data), false).map(function (c) { return c.charCodeAt(0); }))
            : null,
    };
    var serialised = ZilliqaMessage.ProtoTransactionCoreInfo.create(msg);
    return Buffer.from(ZilliqaMessage.ProtoTransactionCoreInfo.encode(serialised).finish());
};
export var isTxReceipt = function (x) {
    return validation.isPlainObject(x) && validation.matchesObject(x, {});
};
export var isTxParams = function (obj) {
    var validator = {
        version: [validation.required(validation.isNumber)],
        toAddr: [validation.required(validation.isAddress)],
        amount: [validation.required(validation.isBN)],
        gasPrice: [validation.required(validation.isBN)],
        gasLimit: [validation.required(validation.isLong)],
        code: [validation.isString],
        data: [validation.isString],
        receipt: [isTxReceipt],
        nonce: [validation.required(validation.isNumber)],
        signature: [validation.required(validation.isSignature)],
    };
    return validation.matchesObject(obj, validator);
};
export var formatOutgoingTx = function (req) {
    var e_1, _a;
    // if batch create transaction, payload is array
    if (Array.isArray(req.payload) &&
        req.payload[0].method === RPCMethod.CreateTransaction &&
        isTxParams(req.payload[0].params[0])) {
        // loop thru batch payloads and format the params
        var payloads = [];
        try {
            for (var _b = __values(req.payload), _c = _b.next(); !_c.done; _c = _b.next()) {
                var txPayload = _c.value;
                var txConfig = txPayload.params[0];
                payloads.push(__assign(__assign({}, txPayload), { params: [
                        __assign(__assign({}, txConfig), { amount: txConfig.amount.toString(), gasLimit: txConfig.gasLimit.toString(), gasPrice: txConfig.gasPrice.toString() }),
                    ] }));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var ret = __assign(__assign({}, req), { payload: payloads });
        return ret;
    }
    // non-batch create transactions
    if (!Array.isArray(req.payload) &&
        req.payload.method === RPCMethod.CreateTransaction &&
        isTxParams(req.payload.params[0])) {
        var txConfig = req.payload.params[0];
        var ret = __assign(__assign({}, req), { payload: __assign(__assign({}, req.payload), { params: [
                    __assign(__assign({}, txConfig), { amount: txConfig.amount.toString(), gasLimit: txConfig.gasLimit.toString(), gasPrice: txConfig.gasPrice.toString() }),
                ] }) });
        return ret;
    }
    return req;
};
export function sleep(ms) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, new Promise(function (resolve) {
                    setTimeout(function () { return resolve(undefined); }, ms);
                })];
        });
    });
}
//# sourceMappingURL=util.js.map